package net.sourceforge.squirrel_sql.plugins.oracle.tokenizer;

import static net.sourceforge.squirrel_sql.fw.sql.OracleSQL.ANON_PROC_EXEC;
import static net.sourceforge.squirrel_sql.fw.sql.OracleSQL.ANON_PROC_EXEC_2;
import static net.sourceforge.squirrel_sql.fw.sql.OracleSQL.CREATE_FUNCTION_SQL;
import static net.sourceforge.squirrel_sql.fw.sql.OracleSQL.CREATE_OR_REPLACE_STORED_PROC;
import static net.sourceforge.squirrel_sql.fw.sql.OracleSQL.CREATE_STORED_PROC;
import static net.sourceforge.squirrel_sql.fw.sql.OracleSQL.NO_SEP_SLASH_SQL;
import static net.sourceforge.squirrel_sql.fw.sql.OracleSQL.SELECT_DUAL;
import static net.sourceforge.squirrel_sql.fw.sql.OracleSQL.SELECT_DUAL_2;
import static net.sourceforge.squirrel_sql.fw.sql.OracleSQL.STUDENTS_NOT_TAKING_CS112;
import static net.sourceforge.squirrel_sql.fw.sql.OracleSQL.UPDATE_TEST;

import java.io.IOException;
import java.util.Arrays;

import junit.framework.TestCase;
import net.sourceforge.squirrel_sql.client.ApplicationManager;
import net.sourceforge.squirrel_sql.client.plugin.PluginQueryTokenizerPreferencesManager;
import net.sourceforge.squirrel_sql.fw.preferences.IQueryTokenizerPreferenceBean;
import net.sourceforge.squirrel_sql.fw.sql.QueryTokenizer;
import net.sourceforge.squirrel_sql.fw.sql.SQLUtil;
import net.sourceforge.squirrel_sql.plugins.oracle.gui.DummyPlugin;
import net.sourceforge.squirrel_sql.plugins.oracle.prefs.OraclePreferenceBean;

public class OracleQueryTokenizerTest extends TestCase {

    static String nullSQL = null;       
    static String tmpFilename = null;
    static boolean removeMultilineComment = true;
    static {
        ApplicationManager.initApplication();        
    }
    
    QueryTokenizer qt = null;
    static int sqlFileStmtCount = 0;
    
    static IQueryTokenizerPreferenceBean _prefs;
    
    public void setUp() throws Exception {
        createSQLFile();
        DummyPlugin plugin = new DummyPlugin();
        PluginQueryTokenizerPreferencesManager prefsManager = new PluginQueryTokenizerPreferencesManager();
        prefsManager.initialize(plugin, new OraclePreferenceBean());
        _prefs = prefsManager.getPreferences();  
    }
    
    public void tearDown() {
        
    }
    
    public void testHasQuery() {
        qt = new OracleQueryTokenizer(_prefs);
        qt.setScriptToTokenize(SELECT_DUAL);
        SQLUtil.checkQueryTokenizer(qt, 1);
        
        qt = new OracleQueryTokenizer(_prefs);
        qt.setScriptToTokenize(SELECT_DUAL_2);
        SQLUtil.checkQueryTokenizer(qt, 1);        
    }

    public void testGenericSQL() {
        String script = SQLUtil.getGenericSQLScript();
        qt = new OracleQueryTokenizer(_prefs);
        qt.setScriptToTokenize(script);
        SQLUtil.checkQueryTokenizer(qt, SQLUtil.getGenericSQLCount());
    }
    
    public void testCreateStoredProcedure() {
        qt = new OracleQueryTokenizer(_prefs);
        qt.setScriptToTokenize(CREATE_STORED_PROC);
        SQLUtil.checkQueryTokenizer(qt, 1);
    }

    public void testCreateOrReplaceStoredProcedure() {
        qt = new OracleQueryTokenizer(_prefs);
        qt.setScriptToTokenize(CREATE_OR_REPLACE_STORED_PROC);
        SQLUtil.checkQueryTokenizer(qt, 1);
    }
    
    public void testHasQueryFromFile() {
        String fileSQL = "@" + tmpFilename + ";\n";
        qt = new OracleQueryTokenizer(_prefs);
        qt.setScriptToTokenize(fileSQL);
        SQLUtil.checkQueryTokenizer(qt, sqlFileStmtCount);
    }

    public void testExecAnonProcedure() {
        qt = new OracleQueryTokenizer(_prefs);
        qt.setScriptToTokenize(ANON_PROC_EXEC);
        SQLUtil.checkQueryTokenizer(qt, 1);
    }    

    public void testExecAnonProcedure2() {
        qt = new OracleQueryTokenizer(_prefs);
        qt.setScriptToTokenize(ANON_PROC_EXEC_2);
        SQLUtil.checkQueryTokenizer(qt, 1);
    }    
    
    public void testNoSepSlash() {
        qt = new OracleQueryTokenizer(_prefs);
        qt.setScriptToTokenize(NO_SEP_SLASH_SQL);
        SQLUtil.checkQueryTokenizer(qt, 2);        
    }    
    
    
    
    private static void createSQLFile() throws IOException {
        if (tmpFilename != null) {
            return;
        }
        String[] sqls = new String[] {
                SELECT_DUAL, 
                UPDATE_TEST,
                CREATE_FUNCTION_SQL,
                CREATE_STORED_PROC,
                CREATE_OR_REPLACE_STORED_PROC,
                ANON_PROC_EXEC,
                SELECT_DUAL,
                STUDENTS_NOT_TAKING_CS112,
                NO_SEP_SLASH_SQL,
        };
        
        tmpFilename = SQLUtil.createSQLFile( Arrays.asList(sqls), true);        
        sqlFileStmtCount = sqls.length;
    }
    
}
